@@ -1,11 +1,12 @@
-from sqlalchemy import text
 from sqlalchemy.sql.expression import _BindParamClause
 from sqlalchemy.ext.compiler import compiles
-from sqlalchemy import schema
-from alembic.ddl import base
-from alembic import util
+from sqlalchemy import schema, text
 from sqlalchemy import types as sqltypes
 
+from ..compat import string_types, text_type, with_metaclass
+from .. import util
+from . import base
+
 class ImplMeta(type):
     def __init__(cls, classname, bases, dict_):
         newtype = type.__init__(cls, classname, bases, dict_)
@@ -15,7 +16,7 @@ class ImplMeta(type):
 
 _impls = {}
 
-class DefaultImpl(object):
+class DefaultImpl(with_metaclass(ImplMeta)):
     """Provide the entrypoint for major migration operations,
     including database-specific behavioral variances.
 
@@ -27,7 +28,6 @@ class DefaultImpl(object):
     bulk inserts.
 
     """
-    __metaclass__ = ImplMeta
     __dialect__ = 'default'
 
     transactional_ddl = False
@@ -50,7 +50,8 @@ class DefaultImpl(object):
         return _impls[dialect.name]
 
     def static_output(self, text):
-        self.output_buffer.write(text + "\n\n")
+        self.output_buffer.write(text_type(text + "\n\n"))
+        self.output_buffer.flush()
 
     @property
     def bind(self):
@@ -59,13 +60,13 @@ class DefaultImpl(object):
     def _exec(self, construct, execution_options=None,
                             multiparams=(),
                             params=util.immutabledict()):
-        if isinstance(construct, basestring):
+        if isinstance(construct, string_types):
             construct = text(construct)
         if self.as_sql:
             if multiparams or params:
                 # TODO: coverage
                 raise Exception("Execution arguments not allowed with as_sql")
-            self.static_output(unicode(
+            self.static_output(text_type(
                     construct.compile(dialect=self.dialect)
                     ).replace("\t", "    ").strip() + self.command_terminator)
         else:
@@ -83,11 +84,14 @@ class DefaultImpl(object):
                         name=None,
                         type_=None,
                         schema=None,
+                        autoincrement=None,
                         existing_type=None,
                         existing_server_default=None,
-                        existing_nullable=None
+                        existing_nullable=None,
+                        existing_autoincrement=None
                     ):
-
+        if autoincrement is not None or existing_autoincrement is not None:
+            util.warn("nautoincrement and existing_autoincrement only make sense for MySQL")
         if nullable is not None:
             self._exec(base.ColumnNullable(table_name, column_name,
                                 nullable, schema=schema,
@@ -119,11 +123,11 @@ class DefaultImpl(object):
                                 existing_nullable=existing_nullable,
                             ))
 
-    def add_column(self, table_name, column):
-        self._exec(base.AddColumn(table_name, column))
+    def add_column(self, table_name, column, schema=None):
+        self._exec(base.AddColumn(table_name, column, schema=schema))
 
-    def drop_column(self, table_name, column, **kw):
-        self._exec(base.DropColumn(table_name, column))
+    def drop_column(self, table_name, column, schema=None, **kw):
+        self._exec(base.DropColumn(table_name, column, schema=schema))
 
     def add_constraint(self, const):
         if const._create_rule is None or \
@@ -174,11 +178,12 @@ class DefaultImpl(object):
             # work around http://www.sqlalchemy.org/trac/ticket/2461
             if not hasattr(table, '_autoincrement_column'):
                 table._autoincrement_column = None
-            self._exec(table.insert(inline=True), multiparams=rows)
+            if rows:
+                self._exec(table.insert(inline=True), multiparams=rows)
 
     def compare_type(self, inspector_column, metadata_column):
 
-        conn_type = inspector_column['type']
+        conn_type = inspector_column.type
         metadata_type = metadata_column.type
 
         metadata_impl = metadata_type.dialect_impl(self.dialect)
@@ -198,9 +203,14 @@ class DefaultImpl(object):
 
     def compare_server_default(self, inspector_column,
                             metadata_column,
-                            rendered_metadata_default):
-        conn_col_default = inspector_column['default']
-        return conn_col_default != rendered_metadata_default
+                            rendered_metadata_default,
+                            rendered_inspector_default):
+        return rendered_inspector_default != rendered_metadata_default
+
+    def correct_for_autogen_constraints(self, conn_uniques, conn_indexes,
+                                        metadata_unique_constraints,
+                                        metadata_indexes):
+        pass
 
     def start_migrations(self):
         """A hook called when :meth:`.EnvironmentContext.run_migrations`
